//+------------------------------------------------------------------------
//
//  Copyright @ Baris TAZE
//
//  File:     GeneratorEx.h
//
//  Content:  Interface and Implementation for CGeneratorEx class
//  
//  Classes:  CGeneratorEx
//            
//  History:  27-May-2002    created by baris taze .  
//
//-------------------------------------------------------------------------

#pragma once

#include "Bit.h"
#include "Population.h"
#include "Generator.h"
#include "GASettings.h"

//////////////////////
template<class TINDIV>
//////////////////////

class CGeneratorEx : public CGenerator<CPopulation<TINDIV, CBit>, TINDIV, CBit>
{
public:
	CGeneratorEx();
	// no destructor
	
	TINDIV* GetIndividualAt( int anIndex );

	void RestoreSettings( CGASettings aSettings );
	CGASettings GetSetings();
	
	bool IsConverged();

protected:
	virtual void PerformCrossOver( TINDIV*& rpPerson, TINDIV*& rpPair );
	virtual bool PerformNaturalSelection( CPopulation<TINDIV, CBit>*& rpCurrentPopulation );	

	CGASettings m_Settings;	
};

//////////////////////
template<class TINDIV>
//////////////////////
CGeneratorEx<TINDIV>::CGeneratorEx()
{

}

//////////////////////
template<class TINDIV>
//template< class TPOP, class TINDIV, class TBIT >
//////////////////////
TINDIV* CGeneratorEx<TINDIV>::GetIndividualAt( int anIndex )
{
	return m_pCurPop->GetIndividual( anIndex );
}

//////////////////////
template<class TINDIV>
//////////////////////
void CGeneratorEx<TINDIV>::PerformCrossOver( TINDIV*& rpPerson, TINDIV*& rpPair )
{
	//see CGenerator::PerformCrossOver
	if( GA_CROSSOVER_ONEPOINT == m_Settings.GetCrossOverType() )
	{		
		int nStartIndex = rand() % rpPerson->GetBitSize(); 
		rpPerson->OnePointCrossOver( rpPair, nStartIndex );
	}
	else if( GA_CROSSOVER_TWOPOINT == m_Settings.GetCrossOverType() )
	{		
		int nStartIndex = rand() % rpPerson->GetBitSize();
		int nEndIndex = nStartIndex + ( rand() % ( rpPerson->GetBitSize() - nStartIndex ) );
		rpPerson->TwoPointCrossOver( rpPair, nStartIndex, nEndIndex );
	}
	else if( GA_CROSSOVER_UNIFORM == m_Settings.GetCrossOverType() )
	{		
		rpPerson->UniformCrossOver( rpPair, (double)m_Settings.GetCrsOvrUnfChance() );
	}
}

//////////////////////
template<class TINDIV>
//////////////////////
bool CGeneratorEx<TINDIV>::PerformNaturalSelection( CPopulation<TINDIV, CBit>*& rpCurrentPopulation )
{
	//see CGenerator::PerformNaturalSelection
	if( GA_NATSEL_ROULETTE == m_Settings.GetNatSelType() )
	{
		double fAvgPerf = rpCurrentPopulation->CalculateAvgPerf();
		rpCurrentPopulation->RouletteWheelProportionate( fAvgPerf );
		return false;
	}

	if( GA_NATSEL_STOCHASTIC == m_Settings.GetNatSelType() )
	{
		double fAvgPerf = rpCurrentPopulation->CalculateAvgPerf();
		rpCurrentPopulation->StochasticRemainderSelection( fAvgPerf );
		return false;
	}
	
	if( GA_NATSEL_GENITOR == m_Settings.GetNatSelType() )
	{
		rpCurrentPopulation->GenitorSelection( m_Settings.GetNatSelGntr() );
		return true;
	}
	
	if( GA_NATSEL_TOURNAMENT == m_Settings.GetNatSelType() )
	{
		rpCurrentPopulation->TournamentSelection( (double)m_Settings.GetNatSelTrnmt() );
		return false;
	}

	return false;
}

//////////////////////
template<class TINDIV>
//////////////////////
void CGeneratorEx<TINDIV>::RestoreSettings( CGASettings aSettings )
{
	m_Settings = aSettings;
}

//////////////////////
template<class TINDIV>
//////////////////////
CGASettings CGeneratorEx<TINDIV>::GetSetings()
{
	return m_Settings;
}

//////////////////////
template<class TINDIV>
//////////////////////
bool CGeneratorEx<TINDIV>::IsConverged()
{
	if( AUTO_STOP_CRITERIA_BIAS == ( AUTO_STOP_CRITERIA_BIAS & m_Settings.GetAStopCriteriaType() ) )
	{
		double fCurBias = GetCurrentBias();
		if( fCurBias >= m_Settings.GetAStopCriteriaBias() ) //bias reached :)
			return true;
	}

	if( AUTO_STOP_CRITERIA_MINUTE == ( AUTO_STOP_CRITERIA_MINUTE & m_Settings.GetAStopCriteriaType() ) )
	{
		CTimeSpan ts = GetTotalTimeSpan();
		long nTotMinutes = ts.GetTotalMinutes();
		if( nTotMinutes >= m_Settings.GetAStopCriteriaMinute() ) //time is over :)
			return true;
	}
	
	if( AUTO_STOP_CRITERIA_ITERATION == ( AUTO_STOP_CRITERIA_ITERATION & m_Settings.GetAStopCriteriaType() ) )
	{		
		int nTotIteration = GetIterationCount();
		if( nTotIteration >= m_Settings.GetAStopCriteriaIteration() ) //iterated so much... :)
			return true;
	}

	if( AUTO_STOP_CRITERIA_REFINEMENT == ( AUTO_STOP_CRITERIA_REFINEMENT & m_Settings.GetAStopCriteriaType() ) )
	{		
		int nWasteIterations = GetSuccessiveUnRefinedIterationCount();
		if( nWasteIterations >= m_Settings.GetAStopCriteriaRefinement() ) //iterated foolishly so much :)
			return true;
	}

	return false;
}